# frozen_string_literal: true

require 'faraday'
require 'openssl'

module BigSession
  # This middleware will add BigSession header to request_headers of outgoing
  # connections done by Faraday
  class FaradayBigSession < ::Faraday::Middleware
    class << self
      def activate
        ::Faraday::Middleware.register_middleware big_session: ::BigSession::FaradayBigSession
      end
    end

    def initialize(app, header_secret = nil)
      @app = app
      @header_secret = header_secret
    end

    def call(env)
      if SessionId.current
        headers = { ::BigSession::BIG_SESSION_HEADER_NAME => SessionId.current, }

        if @header_secret
          sig = OpenSSL::HMAC.hexdigest('sha256', @header_secret, SessionId.current)
          headers[::BigSession::BIG_SESSION_SIGNATURE_HEADER_NAME] = sig
        end

        env[:request_headers].merge!(headers)
      end
      @app.call(env)
    end
  end
end
